@@ -1,8 +1,8 @@
 #[cfg(not(feature = "library"))]
 use cosmwasm_std::entry_point;
 use cosmwasm_std::{
-    attr, to_binary, Binary, BlockInfo, Deps, DepsMut, Env, MessageInfo, Order, Pair, Response,
-    StdError, StdResult, SubMsg,
+    to_binary, Binary, BlockInfo, Deps, DepsMut, Env, MessageInfo, Order, Pair, Response, StdError,
+    StdResult,
 };
 
 use cw0::maybe_addr;
@@ -102,16 +102,10 @@ pub fn execute_mint(
 
     increment_tokens(deps.storage)?;
 
-    Ok(Response {
-        messages: vec![],
-        attributes: vec![
-            attr("action", "mint"),
-            attr("minter", info.sender),
-            attr("token_id", msg.token_id),
-        ],
-        events: vec![],
-        data: None,
-    })
+    Ok(Response::new()
+        .add_attribute("action", "mint")
+        .add_attribute("minter", info.sender)
+        .add_attribute("token_id", msg.token_id))
 }
 
 pub fn execute_transfer_nft(
@@ -123,17 +117,11 @@ pub fn execute_transfer_nft(
 ) -> Result<Response, ContractError> {
     _transfer_nft(deps, &env, &info, &recipient, &token_id)?;
 
-    Ok(Response {
-        messages: vec![],
-        attributes: vec![
-            attr("action", "transfer_nft"),
-            attr("sender", info.sender),
-            attr("recipient", recipient),
-            attr("token_id", token_id),
-        ],
-        events: vec![],
-        data: None,
-    })
+    Ok(Response::new()
+        .add_attribute("action", "transfer_nft")
+        .add_attribute("sender", info.sender)
+        .add_attribute("recipient", recipient)
+        .add_attribute("token_id", token_id))
 }
 
 pub fn execute_send_nft(
@@ -154,17 +142,12 @@ pub fn execute_send_nft(
     };
 
     // Send message
-    Ok(Response {
-        messages: vec![SubMsg::new(send.into_cosmos_msg(contract.clone())?)],
-        attributes: vec![
-            attr("action", "send_nft"),
-            attr("sender", info.sender),
-            attr("recipient", contract),
-            attr("token_id", token_id),
-        ],
-        events: vec![],
-        data: None,
-    })
+    Ok(Response::new()
+        .add_message(send.into_cosmos_msg(contract.clone())?)
+        .add_attribute("action", "send_nft")
+        .add_attribute("sender", info.sender)
+        .add_attribute("recipient", contract)
+        .add_attribute("token_id", token_id))
 }
 
 pub fn _transfer_nft(
@@ -194,17 +177,11 @@ pub fn execute_approve(
 ) -> Result<Response, ContractError> {
     _update_approvals(deps, &env, &info, &spender, &token_id, true, expires)?;
 
-    Ok(Response {
-        messages: vec![],
-        attributes: vec![
-            attr("action", "approve"),
-            attr("sender", info.sender),
-            attr("spender", spender),
-            attr("token_id", token_id),
-        ],
-        events: vec![],
-        data: None,
-    })
+    Ok(Response::new()
+        .add_attribute("action", "approve")
+        .add_attribute("sender", info.sender)
+        .add_attribute("spender", spender)
+        .add_attribute("token_id", token_id))
 }
 
 pub fn execute_revoke(
@@ -216,17 +193,11 @@ pub fn execute_revoke(
 ) -> Result<Response, ContractError> {
     _update_approvals(deps, &env, &info, &spender, &token_id, false, None)?;
 
-    Ok(Response {
-        messages: vec![],
-        attributes: vec![
-            attr("action", "revoke"),
-            attr("sender", info.sender),
-            attr("spender", spender),
-            attr("token_id", token_id),
-        ],
-        events: vec![],
-        data: None,
-    })
+    Ok(Response::new()
+        .add_attribute("action", "revoke")
+        .add_attribute("sender", info.sender)
+        .add_attribute("spender", spender)
+        .add_attribute("token_id", token_id))
 }
 
 pub fn _update_approvals(
@@ -287,16 +258,10 @@ pub fn execute_approve_all(
     let operator_addr = deps.api.addr_validate(&operator)?;
     OPERATORS.save(deps.storage, (&info.sender, &operator_addr), &expires)?;
 
-    Ok(Response {
-        messages: vec![],
-        attributes: vec![
-            attr("action", "approve_all"),
-            attr("sender", info.sender),
-            attr("operator", operator),
-        ],
-        events: vec![],
-        data: None,
-    })
+    Ok(Response::new()
+        .add_attribute("action", "approve_all")
+        .add_attribute("sender", info.sender)
+        .add_attribute("operator", operator))
 }
 
 pub fn execute_revoke_all(
@@ -308,16 +273,10 @@ pub fn execute_revoke_all(
     let operator_addr = deps.api.addr_validate(&operator)?;
     OPERATORS.remove(deps.storage, (&info.sender, &operator_addr));
 
-    Ok(Response {
-        messages: vec![],
-        attributes: vec![
-            attr("action", "revoke_all"),
-            attr("sender", info.sender),
-            attr("operator", operator),
-        ],
-        events: vec![],
-        data: None,
-    })
+    Ok(Response::new()
+        .add_attribute("action", "revoke_all")
+        .add_attribute("sender", info.sender)
+        .add_attribute("operator", operator))
 }
 
 /// returns true iff the sender can execute approve or reject on the contract
@@ -507,23 +466,16 @@ fn query_tokens(
     limit: Option<u32>,
 ) -> StdResult<TokensResponse> {
     let limit = limit.unwrap_or(DEFAULT_LIMIT).min(MAX_LIMIT) as usize;
-    let start_addr = maybe_addr(deps.api, start_after)?;
-    let start = start_addr.map(|addr| Bound::exclusive(addr.as_ref()));
+    let start = start_after.map(Bound::exclusive);
 
     let owner_addr = deps.api.addr_validate(&owner)?;
-    let res: Result<Vec<_>, _> = tokens()
+    let pks: Vec<_> = tokens()
         .idx
         .owner
-        .pks(
-            deps.storage,
-            Vec::from(owner_addr.as_ref()),
-            start,
-            None,
-            Order::Ascending,
-        )
+        .prefix(owner_addr)
+        .keys(deps.storage, start, None, Order::Ascending)
         .take(limit)
         .collect();
-    let pks = res?;
 
     let res: Result<Vec<_>, _> = pks.iter().map(|v| String::from_utf8(v.to_vec())).collect();
     let tokens = res.map_err(StdError::invalid_utf8)?;
@@ -759,15 +711,11 @@ mod tests {
 
         assert_eq!(
             res,
-            Response {
-                attributes: vec![
-                    attr("action", "transfer_nft"),
-                    attr("sender", "venus"),
-                    attr("recipient", "random"),
-                    attr("token_id", token_id),
-                ],
-                ..Response::default()
-            }
+            Response::new()
+                .add_attribute("action", "transfer_nft")
+                .add_attribute("sender", "venus")
+                .add_attribute("recipient", "random")
+                .add_attribute("token_id", token_id)
         );
     }
 
@@ -813,9 +761,9 @@ mod tests {
             token_id: token_id.clone(),
             msg,
         };
-        let expected = SubMsg::new(payload.into_cosmos_msg(target.clone()).unwrap());
+        let expected = payload.into_cosmos_msg(target.clone()).unwrap();
         // ensure expected serializes as we think it should
-        match &expected.msg {
+        match &expected {
             CosmosMsg::Wasm(WasmMsg::Execute { contract_addr, .. }) => {
                 assert_eq!(contract_addr, &target)
             }
@@ -824,16 +772,12 @@ mod tests {
         // and make sure this is the request sent by the contract
         assert_eq!(
             res,
-            Response {
-                messages: vec![expected],
-                attributes: vec![
-                    attr("action", "send_nft"),
-                    attr("sender", "venus"),
-                    attr("recipient", "another_contract"),
-                    attr("token_id", token_id),
-                ],
-                ..Response::default()
-            }
+            Response::new()
+                .add_message(expected)
+                .add_attribute("action", "send_nft")
+                .add_attribute("sender", "venus")
+                .add_attribute("recipient", "another_contract")
+                .add_attribute("token_id", token_id)
         );
     }
 
@@ -868,15 +812,11 @@ mod tests {
         let res = execute(deps.as_mut(), mock_env(), owner, approve_msg).unwrap();
         assert_eq!(
             res,
-            Response {
-                attributes: vec![
-                    attr("action", "approve"),
-                    attr("sender", "demeter"),
-                    attr("spender", "random"),
-                    attr("token_id", token_id.clone()),
-                ],
-                ..Response::default()
-            }
+            Response::new()
+                .add_attribute("action", "approve")
+                .add_attribute("sender", "demeter")
+                .add_attribute("spender", "random")
+                .add_attribute("token_id", token_id.clone())
         );
 
         // random can now transfer
@@ -980,14 +920,10 @@ mod tests {
         let res = execute(deps.as_mut(), mock_env(), owner, approve_all_msg).unwrap();
         assert_eq!(
             res,
-            Response {
-                attributes: vec![
-                    attr("action", "approve_all"),
-                    attr("sender", "demeter"),
-                    attr("operator", "random"),
-                ],
-                ..Response::default()
-            }
+            Response::new()
+                .add_attribute("action", "approve_all")
+                .add_attribute("sender", "demeter")
+                .add_attribute("operator", "random")
         );
 
         // random can now transfer
